/**************************************************************************************
 MAT 259 Final Project: What Do Americans Like to Cook?               
 Tested in Processing 3                                    
 
 Author: Chantal Nguyen                        
 Supervisor: George Legrady                           
 
 Purpose: About 30000 recipes were extracted from epicurious.com, a popular database 
 of recipes collected from American food publications. tf-idf and k-means clustering 
 were used to group the recipes into 15 clusters. SVD and t-SNE were used to reduce 
 dimensionality and facilitate visualization in 3D. Each recipe is shown as a line. 
 The x-y coords of each line correspond to the t-SNE coordinates. The z-coordinate 
 represents the date the recipe was published; the thickness of the line represents 
 the rating of the recipe; and the length of the line represents how many reviews the
 recipe has. The color of the line corresponds to its cluster label. 
 
 Usage: 
 1.  A mouse left-drag will rotate the camera around the subject.
 2.  A right drag will zoom in and out. 
 3.  A middle-drag (command-left-drag on mac) will pan. 
 4.  A double-click restores the camera to its original position. 
 5.  The shift key constrains rotation and panning to one axis or the other.
 6.  Hovering over a line will show the corresponding recipe title.
 **************************************************************************************/

import peasy.*;
import controlP5.*;

PeasyCam cam;
ControlP5 cp5;


PFont f;
PFont f2;

float[] rotations = new float[3];

JSONArray recipeJSON;

ArrayList<recipe> recipes = new ArrayList<recipe>();

float max_x_coord, max_y_coord, min_x_coord, min_y_coord, max_reviews, min_date, max_date;

final int minBoxSize = -200;
final int maxBoxSize = 200;
recipe selectedRecipe = null;

float sliderYear;

color cc1 = #F46C66;
color cc2 = #4C8493;
color cc3 = #FAC856;
color cc4 = #6E987B;
color cc5 = #BB4F24;
color cc6 = #3D89A7;
color cc7 = #CDA91D;
color cc8 = #841F11;
color cc9 = #C7904E;
color cc10 = #335144;
color cc11 = #C38C66;
color cc12 = #D31719;
color cc13 = #3C3B54;
color cc14 = #D28F33;
color cc15 = #133D30;
color cc16 = #CEB3D3;

color[] colors = {cc1, cc2, cc3, cc4, cc5, cc6, cc7, cc8, cc9, cc10, cc11, cc12, cc13, cc14, cc15, cc16};
color c;

void setup() {
  size(1300, 850, P3D);
  smooth();
  cam = new PeasyCam(this, 500);
  cam.setWheelScale(0.1);

  float cameraZ = (height/2.0) / tan(PI*60.0/360.0);
  perspective(PI/3.5, width/(1.1*height), 0.001, cameraZ*10.0); //set near clip to 0.001 to go inside data

  f = createFont("Futura", 24); 
  f2 = createFont("Futura-Bold", 40);  

  recipeJSON = loadJSONArray("epicurious_data.json");

  min_x_coord = 100000;
  min_y_coord = 100000;
  min_date = 100000;
  for (int i=0; i<recipeJSON.size(); i++) {
    recipes.add(new recipe(recipeJSON.getJSONObject(i).getString("hed"), recipeJSON.getJSONObject(i).getFloat("aggregateRating"), recipeJSON.getJSONObject(i).getFloat("reviewsCount"), recipeJSON.getJSONObject(i).getFloat("pubDate"), recipeJSON.getJSONObject(i).getFloat("willMakeAgainPct"), recipeJSON.getJSONObject(i).getFloat("x"), recipeJSON.getJSONObject(i).getFloat("y"), recipeJSON.getJSONObject(i).getInt("label"), recipeJSON.getJSONObject(i).getString("datestr")));
    if (recipeJSON.getJSONObject(i).getFloat("x") > max_x_coord) {
      max_x_coord = recipeJSON.getJSONObject(i).getFloat("x");
    }
    if (recipeJSON.getJSONObject(i).getFloat("x") < min_x_coord) {
      min_x_coord = recipeJSON.getJSONObject(i).getFloat("x");
    }
    if (recipeJSON.getJSONObject(i).getFloat("y") > max_y_coord) {
      max_y_coord = recipeJSON.getJSONObject(i).getFloat("y");
    }
    if (recipeJSON.getJSONObject(i).getFloat("y") < min_y_coord) {
      min_y_coord = recipeJSON.getJSONObject(i).getFloat("y");
    }
    if (recipeJSON.getJSONObject(i).getFloat("reviewsCount") > max_reviews) {
      max_reviews = recipeJSON.getJSONObject(i).getFloat("reviewsCount");
    }
    if (recipeJSON.getJSONObject(i).getFloat("pubDate") > max_date) {
      max_date = recipeJSON.getJSONObject(i).getFloat("pubDate");
    }
    if (recipeJSON.getJSONObject(i).getFloat("pubDate") < min_date) {
      min_date = recipeJSON.getJSONObject(i).getFloat("pubDate");
    }
  }
  
  // slider to set cutoff year
  // all recipes published before cutoff year are hidden
  cp5 = new ControlP5(this);
  cp5.setAutoDraw(false);
  Slider yearSlider = cp5.addSlider("yearSlider")
    .setPosition(10, 100)
    .setRange(1, 1.5)
    .setCaptionLabel("year")
    ;
}

void draw() {
  background(180);
  //rotateY(frameCount*0.0005);
  cam.beginHUD();
  cp5.draw();
  cam.endHUD();
  for (int i=0; i<recipes.size(); i++) {
    if (recipes.get(i).pubdate > sliderYear) {
      float xVal = map(recipes.get(i).x_coord, min_x_coord, max_x_coord, minBoxSize, maxBoxSize);
      float yVal = map(recipes.get(i).y_coord, min_y_coord, max_y_coord, minBoxSize, maxBoxSize);
      float zVal = map(recipes.get(i).pubdate, min_date, max_date, minBoxSize, maxBoxSize);
      pushMatrix();
      translate(xVal, zVal, yVal);
      strokeWeight((int)map(recipes.get(i).rating, 0, 5, 0.1, 7));
      stroke(colors[recipes.get(i).label], (int)map((int)pow(recipes.get(i).makeagain, 4), 0, pow(100, 4), 10, 255));
      //stroke(colors[recipes.get(i).label], (int)map((int)pow(recipes.get(i).makeagain, 1), 0, pow(100, 1), 10, 255));

      line(0, 0, 0, 0, 0 + map(recipes.get(i).review_count, 1, max_reviews, 0, maxBoxSize), 0);
      // comment the above line and uncomment the following to display rounded lines (approximated as dots plotted closely together in a row)
      // however, the interaction will become extremely slow as a result
      //float endpt = map(recipes.get(i).review_count, 1, max_reviews, 0, maxBoxSize);
      //int num_dots = ceil(3*endpt)+1;
      //for (int j = 0; j < num_dots; j++) {
      //point(0, map(j, 0, num_dots-1, 0, endpt), 0);
      //}
      PVector mouse_pos = new PVector(mouseX, mouseY);
      PVector screen_pos = new PVector(screenX(0, 0, 0), screenY(0, 0, 0));
      PVector screen_pos2 = new PVector(screenX(0, 0 + map(recipes.get(i).review_count, 1, max_reviews, 0, maxBoxSize), 0), screenY(0, 0 + map(recipes.get(i).review_count, 1, max_reviews, 0, maxBoxSize), 0));   
      float mouseObjectDist = mouse_pos.dist(screen_pos);
      float mouseObjectDist2 = mouse_pos.dist(screen_pos2);
      popMatrix();

      if ((screen_pos.x-5 <= mouse_pos.x && mouse_pos.x <= screen_pos2.x+5 && screen_pos.y-5 <= mouse_pos.y && mouse_pos.y <= screen_pos2.y+5) || (screen_pos2.x-5 <= mouse_pos.x && mouse_pos.x <= screen_pos.x+5 && screen_pos2.y-5 <= mouse_pos.y && mouse_pos.y <= screen_pos.y+5) || (screen_pos2.x-5 <= mouse_pos.x && mouse_pos.x <= screen_pos.x+5 && screen_pos.y-5 <= mouse_pos.y && mouse_pos.y <= screen_pos2.y+5)|| (screen_pos.x-5 <= mouse_pos.x && mouse_pos.x <= screen_pos2.x+5 && screen_pos2.y-5 <= mouse_pos.y && mouse_pos.y <= screen_pos.y+5)) {
        selectedRecipe = recipes.get(i);
      }
    }
  }
  showHighlightedRecipe(selectedRecipe);
  cam.beginHUD();
  textAlign(LEFT, TOP);
  textFont(f2, 40);
  fill(255, 200);
  text("What Do Americans Like to Cook?", 10, 10);
  textFont(f, 12);
  text("Data from Epicurious.com", 10, 60);
  cam.endHUD();
}



void showHighlightedRecipe(recipe selectedRecipe) {
  if (selectedRecipe != null) {
    float xV = map(selectedRecipe.x_coord, min_x_coord, max_x_coord, minBoxSize, maxBoxSize);
    float yV = map(selectedRecipe.y_coord, min_y_coord, max_y_coord, minBoxSize, maxBoxSize);
    float zV = map(selectedRecipe.pubdate, min_date, max_date, minBoxSize, maxBoxSize); 
    pushMatrix();
    translate(xV, zV + map(selectedRecipe.review_count, 1, max_reviews, 0, maxBoxSize), yV);
    strokeWeight(20);
    fill(255);
    textMode(SHAPE);
    textFont(f, 5);
    pushMatrix();
    rotations = cam.getRotations();
    rotateX(rotations[0]);
    rotateY(rotations[1]);
    rotateZ(rotations[2]);
    text(selectedRecipe.title, 2, 2);
    text(selectedRecipe.datestring, 2, 8);
    text(selectedRecipe.rating + " stars", 2, 14);
    text((int)selectedRecipe.makeagain + "% will make again", 2, 20);

    popMatrix();
    popMatrix();

    // in top left corner, print highlighted label
    cam.beginHUD();
    textAlign(LEFT, BOTTOM);
    textFont(f, 24);
    fill(255);
    text(selectedRecipe.title, 10, height-20);

    textAlign(RIGHT, BOTTOM);
    textFont(f, 14);
    text(selectedRecipe.datestring, width-10, height-60);
    text(selectedRecipe.rating + " stars", width-10, height-45);
    text(selectedRecipe.review_count + " reviews", width-10, height-30);
    text((int)selectedRecipe.makeagain + "% will make again", width-10, height-15);

    cam.endHUD();
  }
}


class recipe {
  String title;
  float rating;
  float review_count;
  float pubdate;
  float makeagain;
  float x_coord;
  float y_coord;
  int label;
  String datestring;

  recipe(String title, float rating, float review_count, float pubdate, float makeagain, float x_coord, float y_coord, int label, String datestring) {
    this.title = title;
    this.rating = rating;
    this.review_count = review_count;
    this.pubdate = pubdate;
    this.makeagain = makeagain;
    this.x_coord = x_coord;
    this.y_coord = y_coord;
    this.label = label;
    this.datestring = datestring;
  }
}

public void yearSlider(float sliderValue) {
  sliderYear = sliderValue;
  println("a slider event. setting background to "+sliderValue);
}
